package cn.fw.dalaran.rpc.oop;

import cn.fw.common.cache.CacheExtender;
import cn.fw.dalaran.common.constants.CacheName;
import cn.fw.dalaran.rpc.oop.dto.CarBrandDTO;
import cn.fw.dalaran.rpc.oop.dto.GroupDTO;
import cn.fw.dalaran.rpc.oop.dto.ShopDTO;
import cn.fw.data.base.domain.common.Message;
import cn.fw.oop.sdk.CarBrandApiService;
import cn.fw.oop.sdk.GroupApiService;
import cn.fw.oop.sdk.ShopApiService;
import cn.fw.oop.sdk.enums.BizTypeEnum;
import cn.fw.oop.sdk.result.CarBrandInfoVo;
import cn.fw.oop.sdk.result.GroupInfoVo;
import cn.fw.oop.sdk.result.ShopInfoVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * OOP RPC
 * <p>
 * create at 2019-07-24
 *
 * @author zhangjj
 */
@Slf4j
@Component
@SuppressWarnings("unchecked")
public class OopRPC {
    /**
     * 缓存扩展器
     */
    private final CacheExtender cacheExtender;
    /**
     * 门店api
     */
    private final ShopApiService shopApiService;
    /**
     * 集团api
     */
    private final GroupApiService groupApiService;
    /**
     * 品牌api
     */
    private final CarBrandApiService brandApiService;

    /**
     * constructor for container
     */
    public OopRPC(final CacheExtender cacheExtender,
                  final ShopApiService shopApiService,
                  final GroupApiService groupApiService,
                  final CarBrandApiService brandApiService) {
        this.cacheExtender = cacheExtender;
        this.shopApiService = shopApiService;
        this.groupApiService = groupApiService;
        this.brandApiService = brandApiService;
    }

    /**
     * 查询所有集团
     *
     * @return 门店列表
     */
    public List<GroupDTO> getAllGroups() {
        final String key = "all";
        List<GroupDTO> groupList = cacheExtender.getCache(CacheName.ALL_GROUPS, key, ArrayList.class);
        if (CollectionUtils.isNotEmpty(groupList)) {
            return groupList;
        }
        final String msg = "调用OOP系统[查询所有集团]";
        try {
            log.info("{}", msg);
            final Message<List<GroupInfoVo>> result = groupApiService.getGroupInfoList();
            if (result.isSuccess()) {
                log.info("{}成功, msg, result:{}", msg, result.getData());
                groupList = Objects.isNull(result.getData())
                    ? Collections.emptyList()
                    : result.getData().stream().map(GroupDTO::with).collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(groupList)) {
                    cacheExtender.putCache(CacheName.ALL_GROUPS, key, groupList);
                }
                return groupList;
            } else {
                throw new RuntimeException(result.getResult());
            }
        } catch (Exception e) {
            log.warn("{}失败", msg, e);
        }
        return Collections.emptyList();
    }

    /**
     * 查询集团在售品牌
     *
     * @return 品牌列表
     */
    public List<CarBrandDTO> getGroupBrands(final Long groupId) {
        List<CarBrandDTO> brandList = cacheExtender.getCache(CacheName.GROUP_BRANDS, groupId, ArrayList.class);
        if (CollectionUtils.isNotEmpty(brandList)) {
            return brandList;
        }
        final String msg = "调用OOP系统[查询集团在售品牌]";
        try {
            log.info("{}, groupId:{}", msg, groupId);
            final Message<List<CarBrandInfoVo>> result = brandApiService.getBrandList(groupId);
            if (result.isSuccess()) {
                log.info("{}成功, groupId:{}, result:{}", msg, groupId, result.getData());
                brandList = Objects.isNull(result.getData())
                    ? Collections.emptyList()
                    : result.getData().stream().map(CarBrandDTO::with).collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(brandList)) {
                    cacheExtender.putCache(CacheName.ALL_GROUPS, groupId, brandList);
                }
                return brandList;
            } else {
                throw new RuntimeException(result.getResult());
            }
        } catch (Exception e) {
            log.warn("{}失败, groupId:{}", msg, groupId, e);
        }
        return Collections.emptyList();
    }

    /**
     * 查询集团下所有门店
     *
     * @param groupId 集团id
     * @return 门店列表
     */
    public List<ShopDTO> getGroupShops(final Long groupId) {
        if (Objects.isNull(groupId)) {
            return Collections.emptyList();
        }
        List<ShopDTO> shopList = cacheExtender.getCache(CacheName.GROUP_SHOPS, groupId, ArrayList.class);
        if (CollectionUtils.isNotEmpty(shopList)) {
            return shopList;
        }
        final String msg = "调用OOP系统[查询集团下所有门店]";
        try {
            log.info("{}, groupId: {}", msg, groupId);
            final Message<List<ShopInfoVo>> result = shopApiService.getShopInfoList(groupId, null, null);
            if (result.isSuccess()) {
                log.info("{}成功, groupId: {}, result: {}", msg, groupId, result.getData());
                shopList = Objects.isNull(result.getData())
                    ? Collections.emptyList()
                    : result.getData().stream().filter(v -> Objects.equals(1, v.getStatus()))
                    .map(ShopDTO::with).collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(shopList)) {
                    cacheExtender.putCache(CacheName.GROUP_SHOPS, groupId, shopList);
                }
                return shopList;
            } else {
                throw new RuntimeException(result.getResult());
            }
        } catch (Exception e) {
            log.warn("{}失败, groupId: {}", msg, groupId, e);
        }
        return Collections.emptyList();
    }

    /**
     * 查询集团售前门店信息
     *
     * @param groupId 集团id
     * @param shopIds 门店id列表
     * @return 门店列表
     */
    public List<ShopDTO> getPreSaleShopList(final Long groupId, final Set<Long> shopIds) {
        if (CollectionUtils.isEmpty(shopIds)) {
            return Collections.emptyList();
        }
        final String msg = "调用OOP系统[查询集团门店信息]";
        try {
            log.info("{}, shopIds:{}", msg, shopIds);
            final Message<List<ShopInfoVo>> result = shopApiService.getShopInfoList(groupId, null, BizTypeEnum.PRE_SALE.getValue());
            if (result.isSuccess()) {
                log.info("{}成功, shopIds:{}, result:{}", msg, shopIds, result.getData());
                List<ShopDTO> shopList = Objects.isNull(result.getData())
                        ? Collections.emptyList()
                        : result.getData().stream()
                        .filter(v -> shopIds.contains(v.getId())).map(ShopDTO::with).collect(Collectors.toList());
                return shopList;
            } else {
                throw new RuntimeException(result.getResult());
            }
        } catch (Exception e) {
            log.warn("{}失败, shopIds:{}", msg, shopIds, e);
        }
        return Collections.emptyList();
    }

}
